#include "FixedOrder.h"

using std::string;
using std::domain_error;

double FixedOrder::pi = M_PI;


void FixedOrder::create() 
{
  as = LHAPDF::alphasPDF(muR);
  lg   = 2*log(muR/mt);
  lgfr = 2*log(muF/muR);
}


/*******************************************************************************	
*									       *	
*		  The Fixed Order results for all partonic channels	       *
*									       *	
*******************************************************************************/

double FixedOrder::FOgg(double rho)
{
  double f = 0.0;	
  if (LO)   { f += PXS_LO_gg(rho) ;}
  if (NLO)  { f += PXS_NLO_NDE_gg(rho) ;}
  if (NNLO) { f += PXS_NNLO_APPROX_gg(rho) ;} // Exact NNLO not yet known. 
  return f * 0.38937966*pow(10,9)*pow(as/mt,2);
}

double FixedOrder::FOqqbar(double rho)
{
  double f = 0.0;	
  if (LO)   { f += PXS_LO_qqbar(rho) ;}
  if (NLO)  { f += PXS_NLO_NDE_qqbar(rho) ;}
  if (NNLO) { f += PXS_NNLO_EXACT_qqbar(rho) ;}  
  return f * 0.38937966*pow(10,9)*pow(as/mt,2);
}

double FixedOrder::FOgq(double rho)
{
  double f = 0.0;	
  if (LO)  { f += 0;}
  if (NLO) { f += PXS_NLO_NDE_gq(rho) ;}
  if (NNLO){ f += PXS_NNLO_APPROX_gq(rho) ;}// No NNLO result known.   
  return f * 0.38937966*pow(10,9)*pow(as/mt,2);
}

double FixedOrder::FOqq(double rho)
{
  double f = 0.0;	
  if (LO)  { f += 0;}
  if (NLO) { f += 0;}
  if (NNLO){ f += PXS_NNLO_qq(rho) ;}
  return f * 0.38937966*pow(10,9)*pow(as/mt,2);
}

double FixedOrder::FOqqprime(double rho)
{
  double f = 0.0;	
  if (LO)  { f += 0;}
  if (NLO) { f += 0;}
  if (NNLO){ f += PXS_NNLO_qqprime(rho) ;}
  return f * 0.38937966*pow(10,9)*pow(as/mt,2);
}

double FixedOrder::FOqqbarprime(double rho)
{
  double f = 0.0;	
  if (LO)  { f += 0;}
  if (NLO) { f += 0;}
  if (NNLO){ f += PXS_NNLO_qqbarprime(rho) ;}
  return f * 0.38937966*pow(10,9)*pow(as/mt,2);
}


/*******************************************************************************	
*									       *	
*		  LO Partonic X-sections		                       *
*									       *	
*******************************************************************************/	

double FixedOrder::PXS_LO_qqbar(double rho)
{
  double f = f0qq(rho);
  return f;
}

double FixedOrder::PXS_LO_gg(double rho)
{
  double f = f0gg(rho);
  return f;
}


/*******************************************************************************	
*									       *	
*		  NLO Partonic X-sections		                       *
*									       *	
*******************************************************************************/	

double FixedOrder::PXS_NLO_NDE_gg(double rho)
{
  double f = f0gg(rho)*(-1.2201878970378643*lgfr*as) 
    + 4*pi*as*(f1gg(rho)+f1ggbar(rho)*(lg+lgfr));
  return f;
}

double FixedOrder::PXS_NLO_NDE_qqbar(double rho)
{
  double f = f0qq(rho)*(-1.2201878970378643*lgfr*as)
    +4*pi*as*(f1qq(rho)+f1qqbar(rho)*(lg+lgfr));
  return f;
}

double FixedOrder::PXS_NLO_NDE_gq(double rho)
{
  double f = 4*pi*as*(f1gq(rho)+f1gqbar(rho)*(lg+lgfr));
  return f;
}


/*******************************************************************************	
 *									       *	
 *	 Exact NNLO results: qq, qqprime and qqbarprime reactions              *
 *									       *	
 *******************************************************************************/	


double FixedOrder::PXS_NNLO_qq(double rho)
{
  double LOG = lg+lgfr;
  double LOG2 = pow(LOG,2);
  
  double f = f20_FIT_qq(rho) 
    + LOG2 * f22_FIT_qqprime(rho) 
    + LOG  * f21_FIT_qq(rho);
  return f * pow(as,2);
}

double FixedOrder::PXS_NNLO_qqprime(double rho)
{
  double LOG = lg+lgfr;
  double LOG2 = pow(LOG,2);
  
  double f = f20_FIT_qqprime(rho)
    + LOG2 * f22_FIT_qqprime(rho) 
    + LOG  * f21_FIT_qqprime(rho);
  return f * pow(as,2);
}

double FixedOrder::PXS_NNLO_qqbarprime(double rho)
{
  double LOG = lg+lgfr;
  double LOG2 = pow(LOG,2);
  
  double f = f20_FIT_qqbarprime(rho)
    + LOG2 * f22_FIT_qqprime(rho) 
    + LOG  * f21_FIT_qqprime(rho);
  return f * pow(as,2);
}

//=======================================================================
// The NNLO functions (no scale-logs) for qq-, qq'- and qqbar'-reactions
//=======================================================================

double FixedOrder::f20_FIT_qq(double rho)
{
  double b = sqrt(1-rho);
  double lrho = log(rho);
  
  double fexp = -0.740558 - 22.8129*pow(b,2) - 0.191648*pow(b,3) 
    - 6.58031*pow(b,4) - 0.537669*pow(b,5) + 31.7872*pow(b,6) 
    - 21.0783*lrho*rho - 3.25313*pow(lrho,2)*rho - 10.8176*lrho*pow(rho,2) 
    + 15.8988*pow(lrho,2)*pow(rho,2) + 8.64557*lrho*pow(rho,3);
  double f = -0.4768323995789214*lrho - pow(b,2)*exp(fexp);	
  return f;
}

double FixedOrder::f20_FIT_qqprime(double rho)
{
  double b = sqrt(1-rho);
  double lrho = log(rho);
  
  double fexp = -0.740558 - 23.4518*pow(b,2) - 0.193073*pow(b,3) 
    - 5.97215*pow(b,4) - 0.541402*pow(b,5) + 31.8227*pow(b,6) 
    - 21.3725*lrho*rho - 3.29162*pow(lrho,2)*rho - 11.1642*lrho*pow(rho,2) 
    + 15.9932*pow(lrho,2)*pow(rho,2) + 8.64746*lrho*pow(rho,3);
  double f = -0.4768323995789214*lrho - pow(b,2)*exp(fexp);
  return f;
}

double FixedOrder::f20_FIT_qqbarprime(double rho)
{
  double b = sqrt(1-rho);
  double lrho = log(rho);
  
  double fexp = -0.740572 - 31.2117*pow(b,2) - 0.31495*pow(b,3) 
    + 15.8601*pow(b,4) - 1.64639*pow(b,5) + 18.9767*pow(b,6) 
    - 19.6977*lrho*rho - 3.16565*pow(lrho,2)*rho - 16.1386*lrho*pow(rho,2) 
    + 12.3828*pow(lrho,2)*pow(rho,2) + 4.17707*lrho*pow(rho,3);
  double f = -0.4768323995789214*lrho - pow(b,2)*exp(fexp);
  return f;
}

//==========================================================================
// The scaling functions ~Log(muF^2/m^2) for qq-, qq'- and qqbar'-reactions
//==========================================================================

double FixedOrder::f22_FIT_qqprime(double rho)// same for all 3 reactions
{
  double b = sqrt(1-rho);
  double lrho = log(rho);
  
  double fexp = -2.20937 - 99.5512*pow(b,2) + 0.394393*pow(b,3) 
    + 92.3169*pow(b,4) + 3.9297*pow(b,5) + 4.81762*pow(b,8) 
    - 43.9374*lrho*rho - 6.31738*pow(lrho,2)*rho - 48.7898*lrho*pow(rho,2) 
    + 25.8588*pow(lrho,2)*pow(rho,2) - 7.30711*lrho*pow(rho,3);	
  double f = -0.157185*pow(b,2) - 0.047419*lrho + pow(b,2)*pow(rho,0.72854)*exp(fexp);
  return f;
}

double FixedOrder::f21_FIT_qqprime(double rho)//same for qq' and qqbar'
{
  double b = sqrt(1-rho);
  double lrho = log(rho);
  
  double fexp = -0.07924228 + 211.8104*pow(b,2) - 0.4464898*pow(b,3) - 196.6873*pow(b,4) 
    - 5.141548*pow(b,5) - 9.448772*pow(b,8) + 90.67104*lrho*rho + 11.02778*pow(lrho,2)*rho 
    + 104.2288*lrho*pow(rho,2) - 55.31094*pow(lrho,2)*pow(rho,2) + 16.62298*lrho*pow(rho,3);
  double f = 1.17852*pow(b,2) + 0.254683*lrho - 1.*pow(b,2)*pow(rho,0.4)*exp(fexp);
  return f;
}

double FixedOrder::f21_FIT_qq(double rho)//for qq only
{
  double b = sqrt(1-rho);
  double lrho = log(rho);
  
  double fexp = -0.07923334 + 209.7125*pow(b,2) - 0.3786914*pow(b,3) - 195.7483*pow(b,4) 
    - 4.500831*pow(b,5) - 9.000056*pow(b,8) + 89.56873*lrho*rho + 10.92725*pow(lrho,2)*rho 
    + 103.6116*lrho*pow(rho,2) - 54.04722*pow(lrho,2)*pow(rho,2) + 16.25028*lrho*pow(rho,3);
  double f = 1.17852*pow(b,2) + 0.254683*lrho - 1.*pow(b,2)*pow(rho,0.4)*exp(fexp);
  return f;
}


/*******************************************************************************	
*									       *	
*	 Approximate NNLO results: gq- and gg-reactions                        *
*        (includes scale terms from LO and NLO)                 	       *   
*									       *	
*******************************************************************************/	

double FixedOrder::PXS_NNLO_APPROX_gq(double rho)
{
  return 0;
}


double FixedOrder::PXS_NNLO_APPROX_gg(double rho)
{
  double tag;
  if (NNLOonORoff) { tag = 1; }// ON
  else { tag = 0; }// OFF
  
  double beta = sqrt(1-rho);
  double lbe = log(beta);
  double a0 = 4608.0,
    a1 = -1894.91,
    a2 = -912.349,
    a3 = 496.3,	
    a4 = 2456.74,
    a5 = 321.137,
    a6 = 68.5471,
    a7 = -8.62261;
  
  double f = a6/pow(beta,2) + (a7 + a5*lbe + a3*pow(lbe,2))/beta 
    + lbe*(a4 + a2*lbe + a1*pow(lbe,2) + a0*pow(lbe,3));
  
  f += Cbargg + 368.0354152962718*lbe*lg + 2606.7683107762314*pow(lbe,2)*lg 
    - 4608.*pow(lbe,3)*lg - 890.988895989886*lbe*pow(lg,2) + 1152.*pow(lbe,2)*pow(lg,2) 
    + 586.9142055529046*lbe*lgfr + 398.7683107762314*pow(lbe,2)*lgfr 
    - 4608.*pow(lbe,3)*lgfr - 677.9777919797721*lbe*lg*lgfr + 2304.*pow(lbe,2)*lg*lgfr 
    + 213.01110401011397*lbe*pow(lgfr,2) + 1152.*pow(lbe,2)*pow(lgfr,2) 
    + (39.63507799167631*lg - 248.15005351310384*lbe*lg 
       - 79.27015598335262*lgfr - 248.15005351310384*lbe*lgfr)/beta 
    + (-170.99993467225795*lg - 104.29134695624361*pow(lg,2) - 842.9932258973022*lgfr 
       - 547.3482065743074*lg*lgfr - 266.7235262847305*pow(lgfr,2))*tag;
  
  return pow((as/(4.0*pi)),2) * f0gg(rho) * f;
}



/*******************************************************************************	
*									       *	
*     NNLO Partonic X-sections	for qqbar-reaction (sum of 4 terms)	       *
*       - exact_threshold term (with its own scale dependence)                 *
*       - fit for the power suppressed terms (including the "constant" term)   *
*       - the contribution with ID final state not icluded above               * 
*         (it is a sum of "pure interference" + qqbar' -> ttbar + qqbar')      *
*       - scale dependent contributions from muF=/=muR originating at LO/NLO   *
*     NOTE: the term ~nl^2 has no threshold, and is known analytically         *
*									       *	
*******************************************************************************/	

double FixedOrder::PXS_NNLO_EXACT_qqbar(double rho)
{
  double f = 
    + PXS_NNLO_THRESHOLD_qqbar(rho) 
    + PXS_NNLO_FIT_qqbar(rho)
    + PXS_NNLO_LO_NLOscales_qqbar(rho)
    + PXS_NNLO_identical_qqbar(rho);
  return f;
}

//================================================================================
// The part of sigma(q qbar -> t tbar q qbar) not included in PXS_NNLO_FIT_qqbar.
// It is a sum of a pure interference term and sigma(q qbar' -> t tbar q qbar').
//================================================================================

double FixedOrder::PXS_NNLO_identical_qqbar(double rho)
{
  double b = sqrt(1-rho);
  double lbe = log(b);
  double lrho = log(rho);
  // the fit of the pure interference contribution:
  double f = -1.26644*pow(b,20)*lbe + 1.53647*pow(b,3)*rho 
    + 10.7411*pow(b,4)*rho + 3.86236*lrho*rho + 0.327143*pow(lrho,2)*rho 
    - 24.3298*pow(b,4)*pow(rho,2) - 21.332*lrho*pow(rho,2) 
    - 10.7669*pow(lrho,2)*pow(rho,2) - 4.50719*pow(b,3)*pow(rho,3) 
    + 15.4975*pow(b,4)*pow(rho,3) + 17.4705*lrho*pow(rho,3) 
    + 2.90068*pow(b,3)*pow(rho,4) - 4.98808*pow(b,4)*pow(rho,4);
  
  f += f20_FIT_qqbarprime(rho);	 
  return f * pow(as,2);
}

//=========================================================================
//   threshold expansion of the NNLO functions f22, f21 and f20 
//   (with exact BORN factored out) from Beneke et al (2009); C2qq=0. 
//   The actual constant is set to zero here and is contained in the fits.
//=========================================================================

double FixedOrder::PXS_NNLO_THRESHOLD_qqbar(double rho)
{
  double nl = 5.0;// number of light flavors
  double b = sqrt(1-rho);
  double lbe = log(b);
  double lbe2 = pow(lbe,2);
  double lbe3 = pow(lbe,3);
  double lbe4 = pow(lbe,4);
  
  double thresh20nl0 = 0.022846306484003143/pow(b,2) 
    - 0.033390549389716966/b + 1.5810883840639216*lbe 
    + (0.3422025061689375*lbe)/b + 6.62693178820539*lbe2 
    - (0.8888888888888888*lbe2)/b - 9.531529857990602*lbe3 
    + 5.76404955831966*lbe4;
  double thresh20nl1 = 0.01168217939778234/b + 0.35320738606771657*lbe 
    - (0.027777777777777776*lbe)/b - 0.5752392118254954*lbe2 + 0.2401687315966525*lbe3;
  double thresh21nl0 = 2.963036815826603 - 0.5208333333333334/b - 5.322595839415953*lbe 
    + (0.4444444444444444*lbe)/b + 11.307833327841358*lbe2 - 5.76404955831966*lbe3;
  double thresh21nl1 = -0.40917468164069626 + 0.041666666666666664/b 
    + 0.46164291174543004*lbe - 0.5403796460924681*lbe2;
  double thresh22nl0 = 1.7154768057697534 - 3.5187082038820767*lbe + 1.441012389579915*lbe2;
  double thresh22nl1 = -0.26328935946926935 + 0.22515818587186173*lbe;
  
  double f20 = thresh20nl0 + nl*thresh20nl1;
  double f21 = thresh21nl0 + nl*thresh21nl1;
  double f22 = thresh22nl0 + nl*thresh22nl1;
  
  double f = f20 + f21*(lg + lgfr) + f22*(pow(lg,2) + 2.*lg*lgfr + pow(lgfr,2));
  
  return pow(as,2) * f0qq(rho) * f;
}


//=========================================================================
//   Scale dependent contributions for muF=/=muR originating at LO and NLO
//=========================================================================

double FixedOrder::PXS_NNLO_LO_NLOscales_qqbar(double rho)
{
  double nl=5.0;
  // the functions fij are normalized as: as^n/m^2
  double f10 = 4*pi*f1qq(rho);
  double f11 = 4*pi*f1qqbar(rho);
  double f00 = f0qq(rho);
  
  double f =
    + f10*(-2.626056561016273*lgfr + 0.15915494309189535*lgfr*nl) 
    + f11*(-2.626056561016273*lg*lgfr - 2.626056561016273*pow(lgfr,2) 
	   + 0.15915494309189535*lg*lgfr*nl + 0.15915494309189535*pow(lgfr,2)*nl) 
    + f00*(-1.2918450914398067*lgfr + 2.298724353885538*pow(lgfr,2) + 0.16042520743370148*lgfr*nl 
	   - 0.2786332550164289*pow(lgfr,2)*nl + 0.008443431970194815*pow(lgfr,2)*pow(nl,2));
  
  return f * pow(as,2);
}


//===========================================================================
//   Fit for the power suppressed terms (contains the qqbar "constant" term)
//   NOTE: the term ~nl^2 has no "threshold", and is known analytically
//===========================================================================

double FixedOrder::PXS_NNLO_FIT_qqbar(double rho)
{
  double nl=5.0;
  // the functions fij are normalized in as^n/m^2
  double f20 = pow(nl,2)*f20nl2_FIT_qqbar(rho) + nl*f20nl1_FIT_qqbar(rho) + f20nl0_FIT_qqbar(rho);
  double f21 = pow(nl,2)*f21nl2_FIT_qqbar(rho) + nl*f21nl1_FIT_qqbar(rho) + f21nl0_FIT_qqbar(rho);
  double f22 = pow(nl,2)*f22nl2_FIT_qqbar(rho) + nl*f22nl1_FIT_qqbar(rho) + f22nl0_FIT_qqbar(rho);
  
  double f = f20 + f21*(lg + lgfr) + f22*(pow(lg,2) + 2.*lg*lgfr + pow(lgfr,2));
  
  return f * pow(as,2);
}

//==========================================================================
//   The fits for qqbar -> ttbar:
//==========================================================================


double FixedOrder::f22nl2_FIT_qqbar(double rho)
{
  return f0qq(rho)/(12.*pow(pi,2));
}

double FixedOrder::f22nl1_FIT_qqbar(double rho)
{
  double b = sqrt(1-rho);
  double lrho = log(rho);
  double f = 0.00239658*b*rho - 0.0220604*pow(b,2)*rho 
    + 0.0188993*pow(b,3)*rho - 0.0131116*lrho*rho 
    - 0.000963099*b*pow(rho,2) + 0.0141569*pow(b,2)*pow(rho,2) 
    - 0.00104753*pow(b,3)*pow(rho,2) - 0.000699993*lrho*pow(rho,2) 
    - 0.0014341*b*pow(rho,3) + 0.00610629*lrho*pow(rho,3) 
    - 0.000214179*lrho*pow(rho,4);
  return f;
}

double FixedOrder::f22nl0_FIT_qqbar(double rho)
{
  double b = sqrt(1-rho);
  double lrho = log(rho);
  double f = 0.191165*pow(b,2) - 13.2537*b*rho 
    + 195.393*pow(b,2)*rho - 139.565*pow(b,3)*rho + 10.4168*lrho*rho 
    + 16.2786*b*pow(rho,2) + 165.851*pow(b,2)*pow(rho,2) 
    + 235.88*pow(b,3)*pow(rho,2) + 213.556*lrho*pow(rho,2) 
    - 3.03232*b*pow(rho,3) + 835.447*pow(b,2)*pow(rho,3) 
    - 90.1477*pow(b,3)*pow(rho,3) + 677.134*lrho*pow(rho,3) 
    + 295.537*lrho*pow(rho,4);
  return f;
}

double FixedOrder::f21nl2_FIT_qqbar(double rho)
{
  double lrho = log(rho);
  return f0qq(rho) * (5 + 3*lrho - 6*log(2))/(18.*pow(pi,2));
}

double FixedOrder::f21nl1_FIT_qqbar(double rho)
{
  double b = sqrt(1-rho);
  double lrho = log(rho);
  double f = - 0.000188644*pow(b,2) + 0.371362*b*rho 
    - 3.14887*pow(b,2)*rho + 3.50366*pow(b,3)*rho 
    + 0.102822*lrho*rho - 0.328962*b*pow(rho,2) 
    + 8.58138*pow(b,2)*pow(rho,2) - 3.93341*pow(b,3)*pow(rho,2) 
    + 1.55362*lrho*pow(rho,2) - 0.0428876*b*pow(rho,3) 
    - 1.31531*pow(b,2)*pow(rho,3) + 0.896763*pow(b,3)*pow(rho,3) 
    + 2.39531*lrho*pow(rho,3) + 0.0208085*lrho*pow(rho,4);
  return f;
}

double FixedOrder::f21nl0_FIT_qqbar(double rho)
{
  double b = sqrt(1-rho);
  double lrho = log(rho);
  double f = -0.720681*pow(b,2) + 46.5167*b*rho - 761.601*pow(b,2)*rho 
    + 503.018*pow(b,3)*rho - 50.2996*lrho*rho - 61.2529*b*pow(rho,2) 
    - 1148.19*pow(b,2)*pow(rho,2) - 914.708*pow(b,3)*pow(rho,2) 
    - 1046.69*lrho*pow(rho,2) + 14.8024*b*pow(rho,3) 
    - 3847.6*pow(b,2)*pow(rho,3) + 377.917*pow(b,3)*pow(rho,3) 
    - 3269.96*lrho*pow(rho,3) - 1386.84*lrho*pow(rho,4);
  return f;
}

double FixedOrder::f20nl2_FIT_qqbar(double rho)
{
  double lrho = log(rho);
  double f = (25 - 3*pow(pi,2) + 30*(lrho - 2*log(2)) 
	      + 9*pow(lrho - 2*log(2),2))/(108.*pow(pi,2));
  return f * f0qq(rho);	
}

double FixedOrder::f20nl1_FIT_qqbar(double rho)
{
  double b = sqrt(1-rho);
  double lrho = log(rho);
  double f = 0.90756*b*rho - 6.75556*pow(b,2)*rho + 9.18183*pow(b,3)*rho 
    + 1.3894*lrho*rho + 0.137881*pow(lrho,2)*rho - 0.99749*b*pow(rho,2) 
    + 27.7454*pow(b,2)*pow(rho,2) - 12.9055*pow(b,3)*pow(rho,2) 
    + 6.13693*lrho*pow(rho,2) - 0.0077383*b*pow(rho,3) 
    - 4.49375*pow(b,2)*pow(rho,3) + 3.86854*pow(b,3)*pow(rho,3) 
    + 8.78276*lrho*pow(rho,3) - 0.380386*pow(b,4)*pow(rho,4) 
    - 0.0504095*lrho*pow(rho,4);
  return f;
}

double FixedOrder::f20nl0_FIT_qqbar(double rho)
{
  double b = sqrt(1-rho);
  double lrho = log(rho);
  double f = -2.32235*b*rho + 44.3522*pow(b,2)*rho - 24.6747*pow(b,3)*rho 
    + 3.11129*lrho*rho + 2.92101*b*pow(rho,2) + 224.311*pow(b,2)*pow(rho,2) 
    + 21.5307*pow(b,3)*pow(rho,2) + 100.125*lrho*pow(rho,2) 
    + 2.05531*b*pow(rho,3) + 945.506*pow(b,2)*pow(rho,3) 
    + 36.1101*pow(b,3)*pow(rho,3) - 176.632*pow(b,4)*pow(rho,3) 
    + 563.1*lrho*pow(rho,3) + 7.68918*pow(b,4)*pow(rho,4) 
    + 568.023*lrho*pow(rho,4);
  return f;
}


/*******************************************************************************	
*									       *	
*	   LO results and the NDE parameterization of the NLO results	       *
*									       *	
*******************************************************************************/	

double FixedOrder::h1(double beta)
{
  double f = -2*gsl_sf_dilog((1 - beta)/2.) 
    + 2*gsl_sf_dilog((1 + beta)/2.) 
    - pow(log((1 - beta)/2.),2) 
    + pow(log((1 + beta)/2.),2);
  return f;
}

double FixedOrder::h2(double beta)
{
  double f = -gsl_sf_dilog((-2*beta)/(1 - beta)) 
    + gsl_sf_dilog((2*beta)/(1 + beta));
  return f;
}

double FixedOrder::f0qq(double rho)
{
  double beta = sqrt(1-rho);
  double f = pi*beta*rho/27.*(2+rho);	
  return f;
}

double FixedOrder::f0qqLeading(double rho)//only the O(beta) term (for testing)
{
  double beta = sqrt(1-rho);
  double f = 0.111111*beta*pi;
  return f;
}

double FixedOrder::f0gg(double rho)
{
  double beta = sqrt(1-rho);
  double f = pi*beta*(rho/192.)
    *((pow(rho,2) + 16*rho+16)
      *log((1+beta)/(1-beta))/beta-28-31*rho);	
  return f;
}

double FixedOrder::f0ggLeading(double rho)//only the O(beta) term (for testing)
{
  double beta = sqrt(1-rho);
  double f = 0.0364583*beta*pi;	
  return f;
}

double FixedOrder::f1qqbar(double rho)
{
  double beta = sqrt(1-rho);
  double nlf = 5;
  double f = (2*rho*log((1 + beta)/(1 - beta)))/(81.*pi) 
    + f0qq(rho)*(-(-127 + 6*nlf)/(72.*pow(pi,2)) 
		 + (2*log(rho/(4.*pow(beta,2))))/(3.*pow(pi,2)));
  return f;
}

double FixedOrder::f1ggbar(double rho)
{
  double beta = sqrt(1-rho);	
  double f = (-181*beta)/(1440.*pi) 
    + (26*beta*rho)/(45.*pi) - (2483*beta*pow(rho,2))/(1920.*pi) 
    + (-rho/(8.*pi) + pow(rho,2)/(16.*pi) - pow(rho,3)/(256.*pi))*h1(beta) 
    + (rho/(8.*pi) + pow(rho,2)/(8.*pi) + pow(rho,3)/(128.*pi))*h2(beta) 
    + ((-3*rho)/(8.*pi) + (33*pow(rho,2))/(128.*pi) 
       + (59*pow(rho,3))/(768.*pi))*log((1 + beta)/(1 - beta)) 
    + (3*f0gg(rho)*log(rho/(4.*pow(beta,2))))/(2.*pow(pi,2));
  return f;
}

double FixedOrder::f1gqbar(double rho)
{
  double beta = sqrt(1-rho);	
  double f = (-181*beta)/(6480.*pi) + (289*beta*rho)/(2160.*pi) 
    - (1319*beta*pow(rho,2))/(25920.*pi) 
    + (-rho/(72.*pi) + pow(rho,2)/(144.*pi))*h1(beta) 
    + ((-17*rho)/(432.*pi) + pow(rho,2)/(128.*pi) 
       + (7*pow(rho,3))/(1728.*pi))*log((1 + beta)/(1 - beta));
  return f;
}

double FixedOrder::f1qq(double rho)
{
  double beta = sqrt(1-rho);
  double nlf = 5;
  double a0 = 0.180899,
    a1 = 0.101949,
    a2 = -0.234371,
    a3 = -0.0109950,
    a4 = -0.0185575,
    a5 = 0.00907527,
    a6 = 0.0160367,
    a7 = 0.00786727;
  
  double f = (a0*beta + a2*pow(beta,3) + a4*pow(beta,5) - pi/432.)*rho 
    + (a1*pow(beta,3) + a3*pow(beta,5) + a5*pow(beta,7) 
       - (41*beta)/(108.*pi))*rho*log(8*pow(beta,2)) 
    + (2*beta*rho*pow(log(8*pow(beta,2)),2))/(27.*pi) 
    + f0qq(rho)*((-5*(-4 + nlf))/(36.*pow(pi,2)) 
		 + ((-4 + nlf)*log(4/rho))/(12.*pow(pi,2))) 
    + a6*beta*rho*log(rho) + a7*beta*rho*pow(log(rho),2);
  return f;
}

double FixedOrder::f1gg(double rho)
{
  double beta = sqrt(1-rho);
  double nlf = 5;	
  double a0 = 0.108068,
    a1 = -0.114997,
    a2 = 0.0428630,
    a3 = 0.131429,
    a4 = 0.0438768,
    a5 = -0.0760996,
    a6 = -0.165878,
    a7 = -0.158246;
  
  double f = a0*beta + a2*pow(beta,3) + (11*pi)/9216. 
    - (beta*(-4 + nlf)*pow(rho,2))/(512.*pi)
    + (a1*pow(beta,3) + a3*pow(beta,5) - (61*beta)/(256.*pi))*log(8*pow(beta,2)) 
    + (7*beta*pow(log(8*pow(beta,2)),2))/(128.*pi) 
    + ((-4 + nlf)*pow(rho,2)*log((1 + beta)/(1 - beta)))/(1024.*pi) 
    + (a6*beta*rho + a4*beta*pow(rho,2))*log(rho) 
    + (a7*beta*rho + a5*beta*pow(rho,2))*pow(log(rho),2);
  return f;
}

double FixedOrder::f1gq(double rho)
{
  double beta = sqrt(1-rho);
  double a0 = 0.0110549,
    a1 = -0.426773,
    a2 = -0.00103876,
    a3 = 0.450626,
    a4 = -0.227229,
    a5 = 0.0472502,
    a6 = -0.197611,
    a7 = -0.0529130;
  
  double f = a1*pow(beta,3) + a3*pow(beta,5) 
    + (a0*pow(beta,3) + a2*pow(beta,5))*log(beta) 
    + (a6*beta*rho + a4*beta*pow(rho,2))*log(rho) 
    + (a7*beta*rho + a5*beta*pow(rho,2))*pow(log(rho),2);
  return f;
}
